
function _ClientStorage (keyPrefix) {
	keyPrefix=(typeof keyPrefix=="undefined"?"":keyPrefix);
	if (keyPrefix=="tunnelx") {
		keyPrefix="";
	}
	if (keyPrefix.length>0) {
		keyPrefix+="_";
	}
	// the prefix will be "" for tunnelx (for backwards compatibility) and something like "tunnelt" for all other skins
	this.keyPrefix=keyPrefix;
	this.localStorage=$.localStorage;
	this._localStorageEnabled=null; // can be null (not yet determined), true, or false
}

var p=_ClientStorage.prototype;


p.isLocalStorageEnabled=function() {
	//console.info("ClientStorage.isLocalStorageEnabled() called");
	var start=new Date().getTime();
	if (ClientStorage._localStorageEnabled==null) {
		try {
			var testKey="k"+new Date().getTime();
			var testValue="v"+new Date().getTime();
			ClientStorage.set(testKey, testValue, true);
			if (ClientStorage.get(testKey, true)!=testValue) {
				ClientStorage._localStorageEnabled=false;
			}
			else {
				ClientStorage._localStorageEnabled=true;							
			}
		}
		catch (e) {
			ClientStorage._localStorageEnabled=false;
		}
		finally {
			// attempt a cleanup after our test, regardless of whether the test succeeded or not:
			try {
				ClientStorage.remove(testKey, true);
			}
			catch (ee) {}
		}
	}
	//console.info("ClientStorage.isLocalStorageEnabled() took "+(new Date().getTime()-start)+"ms");
	return ClientStorage._localStorageEnabled;
};

p.set=function(key, value, skipEnabledCheck) {
	//console.info("ClientStorage.set() called for key "+key);
	var start=new Date().getTime();
	skipEnabledCheck=(typeof skipEnabledCheck=="undefined"?false:skipEnabledCheck);
	if (!skipEnabledCheck && !ClientStorage.isLocalStorageEnabled()) {
		throw new Error("ClientStorage is not enabled for this browser.");
	}
	ClientStorage.localStorage.set(ClientStorage.keyPrefix+key, value);
	//console.info("ClientStorage.set() for key="+key+" took "+(new Date().getTime()-start)+"ms");

};

p.exists=function(key, skipEnabledCheck) {
	var start=new Date().getTime();
	//console.info("ClientStorage.exists() called for key "+key);
	skipEnabledCheck=(typeof skipEnabledCheck=="undefined"?false:skipEnabledCheck);
	if (!skipEnabledCheck && !ClientStorage.isLocalStorageEnabled()) {
		throw new Error("ClientStorage is not enabled for this browser.");
	}
	//console.info("ClientStorage.exists() for key "+key+" took "+(new Date().getTime()-start)+"ms");
	return (ClientStorage.get(key)!=null);
};

p.get=function(key, skipEnabledCheck) {
	//console.info("ClientStorage.get() called for key "+key);
	var start=new Date().getTime();
	skipEnabledCheck=(typeof skipEnabledCheck=="undefined"?false:skipEnabledCheck);
	if (!skipEnabledCheck && !ClientStorage.isLocalStorageEnabled()) {
		throw new Error("ClientStorage is not enabled for this browser.");
	}
	var val=ClientStorage.localStorage.get(ClientStorage.keyPrefix+key);
	//console.info("ClientStorage.get() for key "+key+" took "+(new Date().getTime()-start)+"ms");
	if (typeof val=="undefined") {
		return null;
	}
	return val;
};


p.remove=function(key, skipEnabledCheck) {
	//console.info("ClientStorage.remove() called for key "+key);
	var start=new Date().getTime();
	skipEnabledCheck=(typeof skipEnabledCheck=="undefined"?false:skipEnabledCheck);
	if (!skipEnabledCheck && !ClientStorage.isLocalStorageEnabled()) {
		throw new Error("ClientStorage is not enabled for this browser.");
	}
	
	var ret=ClientStorage.localStorage.remove(ClientStorage.keyPrefix+key);
	//console.info("ClientStorage.remove() for key "+key+" took "+(new Date().getTime()-start)+"ms");
	return ret;
};

p.wipe=function(skipEnabledCheck) {
	//console.info("ClientStorage.wipe() called");
	skipEnabledCheck=(typeof skipEnabledCheck=="undefined"?false:skipEnabledCheck);
	if (!skipEnabledCheck && !ClientStorage.isLocalStorageEnabled()) {
		throw new Error("ClientStorage is not enabled for this browser.");
	}
	var ret=ClientStorage.localStorage.removeAll();
	//console.info("ClientStorage.wipe() took "+(new Date().getTime()-start)+"ms");
	return ret;
};


var ClientStorage=new _ClientStorage(window.skinCode);
