

function _ImageKeyUploader () {
	this.FILE_INPUT_SELECTOR="#fileUploadForm #file";
	this.CORDOVA_FILE_INPUT_SELECTOR="#cordovaFileUri";
	this.UPDATE_INTERVAL_MILLIS=250;

	this.currentPageCode=null; // the current page (e.g. authCreatingUserUploadNewImageKey) on which the user is on the process of choosing a file to upload
	this.uploadFileClickTimeout=null;
	this.postSubmitActionClickHandlerFunctionName=null;
	this.doGetImageKeyHexBytesUploadChecks=false;
	this.allowedPageCodes=new Map(); // we'll use the keys only; basically a set
	this.allowedPageCodes.put("authSecondUserChooseKey", "");
	this.allowedPageCodes.put("authUploadExistingImageKey", "");
	this.allowedPageCodes.put("authCreatingUserUploadNewImageKey", "");
	this.allowedPageCodes.put("authSecondUserUploadNewImageKey", "");
	this.allowedPageCodes.put("authExistingAccountUploadNewDeviceImageKey", "");
	this.lastFilenameOnlyUploaded=null;
}

var p=_ImageKeyUploader.prototype;


/** 
 * Must be called before using ImageKeyUploader to manage file-upload-related input from user.
 */
p.startUseForPage=function(pageCode) {
	if (ImageKeyUploader.allowedPageCodes.get(pageCode)==null) {
		throw new Error("pageCode "+pageCode+" is not in ImageKeyUploader.allowedPageCodes.");
	}
	ImageKeyUploader.reset();
	ImageKeyUploader.currentPageCode=pageCode;
	UiMgr.currentMultipartMode="keyUpload";
	// now, if we're doing the transparent-upload-element thing, we have to write that HTML in dynamically:
	// clear previous instances of this form if we've written them; otherwwise we may end up with duplicte forms on the
	// same page, which will be a nightmare bug:
	UiMgr.clearAllTransparentUploadFileForms();
	if (UiMgr.transparentFormElHtml==null) { // first use -- so populate this value:
		UiMgr.transparentFormElHtml=$("#transparentFileUploadFormTemplate").html().replace(/\_FORMTEMPLATE\_/g, "");
	}
	$("#transparentFileUploadFieldWrapper_"+pageCode).html(UiMgr.transparentFormElHtml);					
	// since in all cases, we are uploading JPGs, we do this (it's removed when the form is submitted), BUT NOTE
	// that this doesn't actually limit the user from choosing non-JPGs; it just shows them alone by default (user
	// can override):
	// NO, PRETTY MUCH USELESS:
	//$("#fileUploadForm #file").attr("accept", "image/jpeg");

};


/**
 * Clears visible and hidden form fields, and properties of ImageKeyUploader that change depending on the currentPageCode).
 */
p.reset=function() {
	// clear the visible filename:
	$("#imagePicker_"+ImageKeyUploader.currentPageCode+" #visibleFilename").val("");
	// Clear the hidden file-submitter form:
	ImageKeyUploader.clearFileUploadFormFields();
	// Null out the context-variable values stored in the ImageKeyUploader object:
	ImageKeyUploader.currentPageCode=null;
	ImageKeyUploader.uploadFileClickTimeout=null;
	ImageKeyUploader.postSubmitActionClickHandlerFunctionName=null;
	UiMgr.currentMultipartMode=null;
};


p.openFileChooser=function() {
	if (ImageKeyUploader.currentPageCode==null) {
		alert("ImageKeyUploader has not been initted for use with a page; call ImageKeyUploader.startUseForPage() first. Exiting from this function now.");
		return;
	}
	ImageKeyUploader.uploadFileClickTimeout=window.setTimeout(ImageKeyUploader.uploadFileFailure, 1000);
	// reset current file if any
	if (!window.useCordovaFilePluginForUploads) {
		$(ImageKeyUploader.FILE_INPUT_SELECTOR).val("");
	}
	else {
		$(ImageKeyUploader.CORDOVA_FILE_INPUT_SELECTOR).val("");
	}
	$("#"+ImageKeyUploader.currentPageCode+" #visibleFilename").val(""); // reset current visible filename, if any
	$("#"+ImageKeyUploader.currentPageCode+" label").show(); // show this (should already be visible)
	if (!window.useCordovaFilePluginForUploads) {
		$(ImageKeyUploader.FILE_INPUT_SELECTOR).click(); // clicking this causes the dialog to open
	}
	else {
		// must manually trigger this click to reset the timeout (runs the same function as the non-cordova case)
		ImageKeyUploader.uploadFileHiddenFieldClick();

		// callback function for when user selects an image
		var cameraSuccess=function(fileUri) {
			$(ImageKeyUploader.CORDOVA_FILE_INPUT_SELECTOR).val(fileUri);
			// trigger the onchange event manually to remain consistent with existing code
			// runs the same function as the non-cordova case
			$(ImageKeyUploader.CORDOVA_FILE_INPUT_SELECTOR).change();
		}

		// set up camera options; note that the defaults are what we need for the rest
		var cameraOptions={};
		cameraOptions.sourceType=Camera.PictureSourceType.SAVEDPHOTOALBUM;
		cameraOptions.quality=100;

		navigator.camera.getPicture(cameraSuccess, null, cameraOptions);
	}
	// We don't do anything else here; if the user chooses a file, the element's onchange handler will fire.
};


p.doFileElementOnchange=function() {
	var currentChosenFile="";
	if (!window.useCordovaFilePluginForUploads) {
		currentChosenFile=$(ImageKeyUploader.FILE_INPUT_SELECTOR).val();
	}
	else {
		currentChosenFile=$(ImageKeyUploader.CORDOVA_FILE_INPUT_SELECTOR).val();
	}
	// filename may be a full file path, or a fake file path, or just a name (for most, it's a name).  We only want
	// the name (and we have to remember different systems have different file separators).  So:
	if (currentChosenFile.length>0) {
		currentChosenFile=currentChosenFile.replace(/\\/g,"/").replace(/\:/g,"/");
		if (currentChosenFile.indexOf("/")>-1) {
			currentChosenFile=currentChosenFile.substring(currentChosenFile.lastIndexOf("/")+1);
		}
		$("#"+ImageKeyUploader.currentPageCode+" label").hide();
	}
	else {
		$("#"+ImageKeyUploader.currentPageCode+" label").show();
	}
	var prettyFormatFilename=UiMgr.prettyFormatFilename(currentChosenFile);
	$("#"+ImageKeyUploader.currentPageCode+" #visibleFilename").val(prettyFormatFilename);
	ImageKeyUploader.lastFilenameOnlyUploaded=prettyFormatFilename;
};



p.clickFileUploadBoxWithErrorTimeout=function() {
	try {
		alert("call should now be made to openFileChooser().");
	}
	catch (__ee) {
		alertJavaScriptErrorIfConfigured(__ee); throw _ee;
	}
};

p.uploadFileFailure=function() {
	DialogMgr.alert("Image-key uploading is not yet supported by your browser.");
};

p.uploadFileHiddenFieldClick=function() {
	// the only reason we track this "click" is to clear the timeout that tells us if the click in the function that 
	// should have opened the file dialog didn't work:
	window.clearTimeout(ImageKeyUploader.uploadFileClickTimeout);
};


p.clearFileUploadFormFields=function() {
	UiMgr._clearFormFields(true);
};

p.getCurrentlyChosenFilename=function() {
	var filename=$("#imagePicker_"+ImageKeyUploader.currentPageCode+" #visibleFilename").val();
	if (typeof filename=="undefined" || filename==null) {
		return "";
	}
	return filename;
};


p.doSubmitAction=function(postSubmitActionClickHandlerFunctionName, validateKey) {
	ImageKeyUploader.postSubmitActionClickHandlerFunctionName=postSubmitActionClickHandlerFunctionName;
	//alert("x "+ImageKeyUploader.postSubmitActionClickHandlerFunctionName)
	var filename=ImageKeyUploader.getCurrentlyChosenFilename();
	if (filename.trim().length==0) {
		DialogMgr.alert("You haven't chosen an image key.  Please try again.");
	}
	else {
		var lcFilename=filename.toLowerCase().trim();
		// nope, we've got android phones that supply jpgs WITHOUT A FILENAME EXTENSION: THANKS, GOOOOOOOOOGLE!!! so, no 
		// morevalidation of this type, so:
		//if (!lcFilename.endsWith(".jpg") && !lcFilename.endsWith(".jpeg")) {

		// remember that whenever we open a dialog before communicating with the server, we have to put that server-
		// communicating code in a timeout; otherwise, on some mobile devices, the dialog will never show up.
		console.log("DEBUG ("+new Date()+"): opening 'this may take several seconds...' dialog");
		DialogMgr.openBlockingModalDialog("One moment (this may take several seconds)...", UiMgr.expectedAuthTimeMillis);
		setTimeout(function(){ImageKeyUploader.uploadImageKeyFile(validateKey);},100);
		//console.log("DEBUG: timeout set to uploadImageKeyFile...");


	}
};

p.uploadImageKeyFile=function(validateKey) {	
	if (typeof validateKey=="undefined") {
		throw new Error("validateKey not specified.");
	}
	// start the upload (remember, all this is is us getting the hex bytes version of the image (the first 40K), which
	// will be passed to uploadImageKeyFileSuccessCallback(), which will call the ClickHandlers function specified 
	// by the caller, passing it the hex bytes (unless there's a failure, in which case uploadImageKeyFileFailureCallback()
	// will be called, with an error message to display.
	console.log("DEBUG ("+new Date()+"): about to startGetImageKeyHexBytesUploadChecks...");
	ImageKeyUploader.startGetImageKeyHexBytesUploadChecks();
	console.info("DEBUG ("+new Date()+"): about to start asynchronous upload with handler "+UiMgr.handlerKeys.GET_IMAGE_KEY_HEX_BYTES+"...");
	CommMgr.startAsynchronousAjaxFileUpload(CommMgr.getControlServer(), UiMgr.handlerKeys.GET_IMAGE_KEY_HEX_BYTES, "?_multipartHandlerKey="+UiMgr.handlerKeys.GET_IMAGE_KEY_HEX_BYTES+"&validateNewKey="+validateKey);	
};


p.startGetImageKeyHexBytesUploadChecks=function() {
	//console.info("in ImageKeyUploader.startGetImageKeyHexBytesUploadChecks()...");
	ImageKeyUploader.doGetImageKeyHexBytesUploadChecks=true;
	ImageKeyUploader.doGetImageKeyHexBytesUploadCheck();
};

p.stopGetImageKeyHexBytesUploadChecks=function() {
	//console.info("in ImageKeyUploader.stopGetImageKeyHexBytesUploadChecks()...");
	ImageKeyUploader.doGetImageKeyHexBytesUploadChecks=false;
};

p.doGetImageKeyHexBytesUploadCheck=function() {
	console.log("DEBUG ("+new Date()+"): call made to doGetImageKeyHexBytesUploadCheck...");
	try {
		var start=new Date().getTime();
		if (ImageKeyUploader.doGetImageKeyHexBytesUploadChecks) {
			console.log("DEBUG ("+new Date()+"): making async call to checker...");

			CommMgr.initiateAjaxRequest(CommMgr.getControlServer(), UiMgr.handlerKeys.CHECK_FOR_GET_IMAGE_KEY_HEX_UPDATES, {}, true, 
					// Success cb:
					function(response) {
						console.log("DEBUG ("+new Date()+"): in success CB, asynchronous call to checker done...");
						var responseUndefined=(typeof response=="undefined");
						if (responseUndefined) {
							console.warn("in success CB, response undefined (AJAX call failed); will try again...");
						}
						if (!responseUndefined && response!=null && response.success && response.status!="unknown") {
							if (response.status=="failure") {
								ImageKeyUploader.doGetImageKeyHexBytesUploadChecks=false;
								var failureMsg;
								if (typeof response.failureDetailsMessage != "undefined" && response.failureDetailsMessage!=null) {
									failureMsg=response.failureDetailsMessage;
								}
								else {
									failureMsg="There was a problem; please try again.";
				
								}
								ImageKeyUploader.uploadImageKeyFileFailureCallback(failureMsg);	
								return;
							}
							else if (response.status=="success") {
								ImageKeyUploader.doGetImageKeyHexBytesUploadChecks=false;
								DialogMgr.closeBlockingModalDialog();
								ImageKeyUploader.uploadImageKeyFileSuccessCallback(response.leadingBytesHex);
								return;
							}
						}
						$("#fileUploadForm #filename").val("");
						console.log("DEBUG ("+new Date()+"): in success CB, about to setTimeout for doGet... again (with millis delay="+ImageKeyUploader.UPDATE_INTERVAL_MILLIS+"...");
						setTimeout(ImageKeyUploader.doGetImageKeyHexBytesUploadCheck, ImageKeyUploader.UPDATE_INTERVAL_MILLIS);
								
					},  
					// Error cb:
					function() {
						console.log("DEBUG ("+new Date()+"): in error CB, about to setTimeout for doGet... again (with millis delay="+ImageKeyUploader.UPDATE_INTERVAL_MILLIS+"...");
						setTimeout(ImageKeyUploader.doGetImageKeyHexBytesUploadCheck, ImageKeyUploader.UPDATE_INTERVAL_MILLIS);						
					},  
					1500);
		}	
	}
	catch (__ee) {
		alertJavaScriptErrorIfConfigured(__ee); throw _ee;
	}
};

/*

p.doGetImageKeyHexBytesUploadCheck=function() {
	console.log("DEBUG ("+new Date()+"): call made to doGetImageKeyHexBytesUploadCheck...");
	try {
		var start=new Date().getTime();
		if (ImageKeyUploader.doGetImageKeyHexBytesUploadChecks) {
			console.log("DEBUG ("+new Date()+"): making synchronous call to checker...");

			var response=CommMgr.initiateAjaxRequest(CommMgr.getControlServer(), UiMgr.handlerKeys.CHECK_FOR_GET_IMAGE_KEY_HEX_UPDATES, {}, false, null,  null,  500);
			console.log("DEBUG ("+new Date()+"): synchronous call to checker done...");
			var responseUndefined=(typeof response=="undefined");
			if (responseUndefined) {
				console.warn("response undefined (AJAX call failed); will try again...");
			}
			if (!responseUndefined && response!=null && response.success && response.status!="unknown") {
				if (response.status=="failure") {
					ImageKeyUploader.doGetImageKeyHexBytesUploadChecks=false;
					var failureMsg;
					if (typeof response.failureDetailsMessage != "undefined" && response.failureDetailsMessage!=null) {
						failureMsg=response.failureDetailsMessage;
					}
					else {
						failureMsg="There was a problem; please try again.";
	
					}
					ImageKeyUploader.uploadImageKeyFileFailureCallback(failureMsg);	
					return;
				}
				else if (response.status=="success") {
					ImageKeyUploader.doGetImageKeyHexBytesUploadChecks=false;
					DialogMgr.closeBlockingModalDialog();
					ImageKeyUploader.uploadImageKeyFileSuccessCallback(response.leadingBytesHex);
					return;
				}
			}
			$("#fileUploadForm #filename").val("");
			console.log("DEBUG ("+new Date()+"): about to setTimeout for doGet... again (with millis delay="+ImageKeyUploader.UPDATE_INTERVAL_MILLIS+"...");
			setTimeout(ImageKeyUploader.doGetImageKeyHexBytesUploadCheck, ImageKeyUploader.UPDATE_INTERVAL_MILLIS);
		}	
		console.info("ImageKeyUploader.doGetImageKeyHexBytesUploadCheck() completed in "+(new Date().getTime()-start)+"");
	}
	catch (__ee) {
		alertJavaScriptErrorIfConfigured(__ee); throw _ee;
	}
};


 */

p.uploadImageKeyFileSuccessCallback=function(keyBytes) {
	DialogMgr.closeBlockingModalDialog();
	//alert("x "+ImageKeyUploader.postSubmitActionClickHandlerFunctionName)
	ClickHandlers[ImageKeyUploader.postSubmitActionClickHandlerFunctionName](keyBytes);
	ImageKeyUploader.reset();
};

p.uploadImageKeyFileFailureCallback=function(errorMessage) {
	DialogMgr.closeBlockingModalDialog();
	DialogMgr.alert(errorMessage);
	ImageKeyUploader.clearFileUploadFormFields();
};








var ImageKeyUploader=new _ImageKeyUploader();
