

//MobileDialog object; only used for mobile (app and web) -- an instance of this becomes DialogMgr.mobileDialog (for mobiles only)
function MobileDialog () {
	// when a mobile dialog is open, these hold references to Functions which will be called when the OK or Cancel buttons are clicked (if they're visible at all)
	this.currentTrueCallback=function(){};
	this.currentFalseCallback=function(){};
	this.progressBarTimeout=null;
	this.currentProgressBarStep=0;
	this.progressBarNumSteps=6; // this is hardcoded because if we change the number of dots (which is driven by our PIN size), there'll be a lot more changes than just here to deal with that.  Sorry.

}

var p=MobileDialog.prototype;



/**
 * @param message The message, which may contain html markup
 * @param buttonNamesArray An array of button names (e.g. ["OK","Cancel"]; if empty, no buttons will show, meaning the user can't close the dialog, meaning you have to; if unspecified or null, taken to be []
 * @param buttonCallbacksrray An array of functions to be called when each of the buttons is clicked (must, then, be the same length as buttonNamesArray); if unspecified or null, taken to be []
 * @param expectedDisplayDurationMillis If and only if this is a buttonless dialog, the number of milliseconds (if any) which we EXPECT the dialog to be open.  (However, the dialog must still be closed by the caller.)  If specified, a progress bar will show. If unspecified, taken to be 0; 0 indicates that no progress bar will be displayed.
 */
p._openMobileModalDialog=function(message, buttonNamesArray, buttonCallbacksArray, expectedDisplayDurationMillis) {
	//DialogMgr.mobileDialog._stopProgressBar();
	//alert("message="+message+" , expectedDisplayDurationMillis="+expectedDisplayDurationMillis)
	if (typeof message=="undefined" || message==null) {
		message=""; // why not.
	}
	if (typeof buttonNamesArray=="undefined" || buttonNamesArray==null) {
		buttonNamesArray=[];
	}
	if (typeof buttonCallbacksArray=="undefined" || buttonCallbacksArray==null) {
		buttonCallbacksArray=[];
	}
	if (buttonNamesArray.length!=buttonCallbacksArray.length) {
		throw new Error("buttonNamesArray and buttonCallbacksArray don't have the same length.");
	}
	if (buttonNamesArray.length>2) {
		throw new Error("buttonNamesArray contains more than 2 buttons; this array (and buttonCallbacksArray) must have length 0 (no buttons), 1 (an alert()), or 2 (a confirm()).");
	}
	if (typeof expectedDisplayDurationMillis=="undefined" || expectedDisplayDurationMillis==null) {
		expectedDisplayDurationMillis=0; // i.e., show no progress bar
	}
	var showProgressBar=(expectedDisplayDurationMillis>0);
	if (buttonNamesArray.length>0 && showProgressBar) {
		throw new Error("A non-zero expectedDisplayDurationMillis was specified but this dialog has buttons; this is not allowed.");
	}

	
	var centerText=(buttonNamesArray.length==0);
	var buttonName;
	var buttonCallback;
	var dialogHolder=$("#mobileModalDialog");
	var messageHolder=$("#mobileModalDialog #messageTd #message");
	var buttonHolder=$("#mobileModalDialog #buttonsTd #buttonWrapper");
	var trueButton=$("#mobileModalDialog #buttonsTd #buttonWrapper #trueButton"); // OK button (if any)
	var falseButton=$("#mobileModalDialog #buttonsTd #buttonWrapper #falseButton"); // Cancel button (if any)
	var progressBar=$("#mobileModalDialog #progressBarTd #barWrapper"); // the progress bar (if any), which is a series of six dots
	
	dialogHolder.hide(); // hide the entire dialog (not likely to be visible anyway, but whatever)
	
	// determine what buttons we're showing:
	var hasTrueButton;
	var hasFalseButton;
	if (buttonNamesArray.length==0) {
		hasTrueButton=false;
		hasFalseButton=false;		
	}
	else if (buttonNamesArray.length==1) {
		hasTrueButton=true;
		hasFalseButton=false;
	}
	else { // else length is 2:
		hasTrueButton=true;
		hasFalseButton=true;
	}
	// note that if length is 0, we have no buttons; this will just be a blocking modal dialog, which the app must make sure to close itself.
	//alert("expectedDisplayDurationMillis="+expectedDisplayDurationMillis+" , showProgressBar="+showProgressBar)
	// now set up the dialog, and then open it:
	if (showProgressBar) {
		progressBar.show(); // note: won't show until the wrapping dialog is shown

		DialogMgr.mobileDialog._startProgressBar(expectedDisplayDurationMillis);
	}
	else {
		progressBar.hide();
	}
	if (hasTrueButton) {
		trueButton.button({label:buttonNamesArray[0]==null || buttonNamesArray[0].length==0?"ok":buttonNamesArray[0].toLowerCase()});
		DialogMgr.mobileDialog.currentTrueCallback=(buttonCallbacksArray[0]==null?function(){}:buttonCallbacksArray[0]);
		trueButton.show();
	}
	else {
		trueButton.button({label:"&nbsp;"});
		DialogMgr.mobileDialog.currentTrueCallback=function(){};
		trueButton.hide();		
	}
	if (hasFalseButton) {
		falseButton.button({label:buttonNamesArray[1]==null || buttonNamesArray[1].length==0?"cancel":buttonNamesArray[1].toLowerCase()});
		DialogMgr.mobileDialog.currentFalseCallback=(buttonCallbacksArray[1]==null?function(){}:buttonCallbacksArray[1]);
		falseButton.show();
	}
	else {
		falseButton.button({label:"&nbsp;"});
		DialogMgr.mobileDialog.currentFalseCallback=function(){};
		falseButton.hide();		
	}
	if (centerText) {
		message="<div style=\"text-align:center;\">"+message+"</div>";
	}
	messageHolder.html(message);
	//if (mobileApp) {
	//	BackbuttonFunctionStack.add(DialogMgr.mobileDialog.closeMobileModalDialog);
	//}
	//dialogHolder.show("fade", 250);
	//alert("DialogMgr.mobileDialog.currentTrueCallback="+DialogMgr.mobileDialog.currentTrueCallback)
	dialogHolder.show();
};



p._openMobileSettingsDialog=function() {
	var dialogHolder=$("#mobileSettingsDialog");
	//BackbuttonFunctionStack.add(DialogMgr.mobileDialog.closeMobileSettingsDialog);
	//dialogHolder.show("fade", 250);
	//alert("DialogMgr.mobileDialog.currentTrueCallback="+DialogMgr.mobileDialog.currentTrueCallback)
	dialogHolder.show();
};



p._openMobileHelpDialog=function() {
	var dialogHolder=$("#mobileHelpDialog");	
	//BackbuttonFunctionStack.add(DialogMgr.mobileDialog.closeMobileHelpDialog);
	//dialogHolder.show("fade", 250);
	//alert("DialogMgr.mobileDialog.currentTrueCallback="+DialogMgr.mobileDialog.currentTrueCallback)
	dialogHolder.show();
};









p._openMobilePageSpecificHelpDialog=function(message) {
	if (typeof message=="undefined" || message==null) {
		message=""; // why not.
	}
	
	var dialogHolder=$("#mobilePageSpecificHelpDialog");
	var messageHolder=$("#mobilePageSpecificHelpDialog #messageTd #message");
	var buttonHolder=$("#mobilePageSpecificHelpDialog #buttonsTd #buttonWrapper");
	var trueButton=$("#mobilePageSpecificHelpDialog #buttonsTd #buttonWrapper #trueButton"); // OK button 
	
	dialogHolder.hide(); // hide the entire dialog (not likely to be visible anyway, but whatever)

	trueButton.button({label:"ok"});
	trueButton.show();

	messageHolder.html(message);
	//BackbuttonFunctionStack.add(DialogMgr.mobileDialog.closeMobilePageSpecificHelpDialog);
	dialogHolder.show();
};





p._startProgressBar=function(expectedDisplayDurationMillis) {
	DialogMgr.mobileDialog.progressBarTimeout=setTimeout(DialogMgr.mobileDialog._doProgressBarStep, expectedDisplayDurationMillis/DialogMgr.mobileDialog.progressBarNumSteps);
};



p._stopProgressBar=function(expectedDisplayDurationMillis) {
	if (DialogMgr.mobileDialog.progressBarTimeout!=null) {
		clearTimeout(DialogMgr.mobileDialog.progressBarTimeout);
		DialogMgr.mobileDialog.progressBarTimeout=null;
		DialogMgr.mobileDialog.currentProgressBarStep=0;
		DialogMgr.mobileDialog._updateVisibleProgressBar();
	}
};







p._doProgressBarStep=function() {
	DialogMgr.mobileDialog.currentProgressBarStep++;
	DialogMgr.mobileDialog._updateVisibleProgressBar();
	if (DialogMgr.mobileDialog.currentProgressBarStep==DialogMgr.mobileDialog.progressBarNumSteps) {
		clearTimeout(DialogMgr.mobileDialog.progressBarTimeout);
		DialogMgr.mobileDialog.progressBarTimeout=null;
	}
	else {
		DialogMgr.mobileDialog.progressBarTimeout=setTimeout(DialogMgr.mobileDialog._doProgressBarStep);
	}
};


p._updateVisibleProgressBar=function() {
	for (var i=0; i<DialogMgr.mobileDialog.progressBarNumSteps; i++) {
		var showOrHide=(i>=DialogMgr.mobileDialog.currentProgressBarStep?"show":"hide");
		$("#mobileModalDialog #progressBarTd #onDotImg"+i)[showOrHide]();
	}
};






//Called when OK or Cancel buttons are clicked; may also be called from anywhere else (buttonless modals must be closed this way) -- buttonId is "trueButton", "falseButton", or null (if no button was clicked, as when we close a buttonless dialog)
p.closeMobileModalDialog=function(buttonId) {
	//if (mobileApp) {
	//	MobileDevice.clearBackbuttonFunction();
	//}
	// note that we don't clear any of the button values, or even hide them; we just hide the wrapping dialog; all showing/hiding
	// happens next time _openMobileModalDialog() is called:
	//$("#mobileModalDialog").hide("fade", 325);
	$("#mobileModalDialog").hide();
	var callback=null;
	if (typeof buttonId!="undefined" && buttonId!=null) {
		// we have a callback to do:
		if (buttonId=="trueButton") {
			callback=DialogMgr.mobileDialog.currentTrueCallback;
		}
		else if (buttonId=="falseButton") {
			callback=DialogMgr.mobileDialog.currentFalseCallback;
		}
		callback();
	}
	// reset these, just so they don't cause any trouble:
	DialogMgr.mobileDialog.currentTrueCallback=function(){};
	DialogMgr.mobileDialog.currentFalseCallback=function(){};
};





p.closeMobileSettingsDialog=function() {
	//MobileDevice.clearBackbuttonFunction();
	$("#mobileSettingsDialog").hide();
};


p.closeMobileHelpDialog=function() {
	//MobileDevice.clearBackbuttonFunction();
	$("#mobileHelpDialog").hide();
};


p.closeMobilePageSpecificHelpDialog=function() {
	//if (mobileApp) {
	//	MobileDevice.clearBackbuttonFunction();
	//}
	// note that we don't clear any of the button values, or even hide them; we just hide the wrapping dialog; all showing/hiding
	// happens next time _openMobilePageSpecificHelpDialog() is called:
	//$("#mobilePageSpecificHelpDialog").hide("fade", 325);
	$("#mobilePageSpecificHelpDialog").hide();
};





