function _NotificationMgr() {
	this.showPleaseWaitDialog=true;
	this.enableNotifications=true;
	this.notificationHelpText="We recommend that you allow Tunnel X to send you a notification when a friend sends you a message. Notifications do not contain nicknames or tunnel names. (If you uncheck this box, you can enable this feature later, in Settings.)";
}

var p=_NotificationMgr.prototype;

/**
 * Initialize the NotificationMgr object and register the client device.
 */
p.init=function(showPleaseWaitDialog) {
	if (typeof showPleaseWaitDialog!="undefined") {
		NotificationMgr.showPleaseWaitDialog=false;
    }
	NotificationMgr.registerDevice();
};

/**
 * Sends the client's uuid to our server.
 */
p.sendUuidToServer=function(clientUuid) {
    if (!window.enablePushNotifications || !mobileApp) {
        return;
    }
    var os=null;
    var uuid=null;
    try {
        if (device.platform.toLowerCase().startsWith("ios")) {
            os="ios";
        }
        if (device.platform.toLowerCase().startsWith("android")) {
            os="android";
        }
        uuid=clientUuid;
    }
    catch (e) {}
    os=(typeof os=="undefined")?null:os;
    uuid=(typeof uuid=="undefined")?null:uuid;
    // regardless of anything:
    if (localMobileAppDebug) {
        os="webDebugOs";
        uuid="webDebugUuid";
    }
    if (os==null || uuid==null) {
        return;
    }

    if (NotificationMgr.showPleaseWaitDialog) {
        DialogMgr.openBlockingModalDialog("One moment please...", UiMgr.expectedAuthTimeMillis);
    }
    setTimeout(function(){
        var dataMap={};
        try {
            // The handler we're about to talk to requires that we get a nonce from the server, generate our
            // own nonce, and hash them with the uuid, then set those values in the form, so:
           // CommMgr.addNonceAndHashToAjaxDataObject(UiMgr.handlerKeys.SEND_UUID, dataMap, uuid);
        }
        catch (e) {
            // Didn't work; we'll swallow this one; don't want to keep user from logging in on acct of UUID issues --
            console.warn("failed to set nonce for SEND_UUID call; returning.");
            return;
        }
        // Send the UUID:
        dataMap.uuid=uuid;
        dataMap.os=os;
        try {
            CommMgr.initiateAjaxRequest(CommMgr.getControlServer(), UiMgr.handlerKeys.SEND_UUID, dataMap, false);
        }
        catch (e) {
            // Didn't work; we'll swallow this one; don't want to keep user from logging in on acct of UUID issues, although we will log the error:
            console.warn("failed to send UUID.");
        }
        if (NotificationMgr.showPleaseWaitDialog) {
            DialogMgr.closeBlockingModalDialog();
        }
    }, 100);
};

/**
 * Temporarily store whether we want to enable notifications during account creation.
 */
p.setNotifications=function(enable) {
	NotificationMgr.enableNotifications=enable;
};

/**
 * Shows a help alert describing how notifications work.
 */
p.showHelp=function() {
	setTimeout(function() {
		DialogMgr.alert(NotificationMgr.notificationHelpText);
	}, 50);
}

/**
 * Retrieves the necessary credentials to register devices.
 */
p.getRegistrationCredentials=function() {
	var response=null;
	var data={};
	data.os=appOs;
	response=CommMgr.initiateAjaxRequest(CommMgr.getControlServer(), UiMgr.handlerKeys.PUSH_NOTIFICATION_REGISTRATION, data, false);
	if (typeof response.success=="undefined" || response.success!=true) {
		throw new Error("Couldn't retrieve device registration credentials: "+response.errorMsg);
	}
	return response.gcmSenderId; 
};

/**
 * Registers a client device to receive push notifications.
 */
p.registerDevice=function() {
	try {
		var registrationOptions=null;
		if (appOs=="android") {
			registrationOptions={
				"senderID": NotificationMgr.getRegistrationCredentials(),
				"ecb": "NotificationMgr.onNotificationGCM"
			};
		}
		else if (appOs=="ios") {
			registrationOptions={
				"badge": "true",
				"sound": "true",
				"alert": "true",
				"ecb": "NotificationMgr.ecbonNotificationAPN"
			};
		}
		else {
			throw new Error("Couldn't determine device platform");
		}
		pushNotification.register(NotificationMgr.registerSuccess, NotificationMgr.registerError, registrationOptions);
	}
	catch(__ee) {
		alertJavaScriptErrorIfConfigured(__ee); throw _ee;
	}
};

/**
 * Callback function for when a device receives a push notification from GCM.
 */
p.onNotificationGCM=function(e) {
	switch (e.event) {
		case "registered":
			if (e.regid.length>0) {
				// android devices are registered here
				NotificationMgr.sendUuidToServer(e.regid);
			}
			break;

		case "message":
			/* COMMENTING OUT since we don't do anyhring here for now
			if (e.foreground) {
				// triggers immediately if we are currently in the app
			}
			else {
				// these are background notifications and only trigger after the notification is clicked
				if (e.coldstart) {
					// app was previously closed
				}
				else {
					// app was running in the background
				}
			}
			*/
			break;

		case "error":
			break;

		default:
			break;
	}
};

/**
 * Callback function for when a device receives a push notification from APN.
 */
p.onNotificationAPN=function(e) {
	// commented out code we dont' use; uncomment when we wanna use
	//if (e.alert) {
	//}
	if (e.sound) {
		var snd=new Media(event.sound);
		snd.play();
	}
	//if (e.badge) {
	//}
};

/**
 * Callback function for when the device was successfully registered.
 */
p.registerSuccess=function(result) {
	if (appOs=="ios") {
		// ios devices are registered here
		NotificationMgr.sendUuidToServer(result);
	}
};

/**
 * Callback function for when there was an error registering the device.
 */
p.registerError=function(error) {
	throw new Error("Couldn't register device for push notifications");
};

if (mobileApp) {
	var NotificationMgr=new _NotificationMgr();
	var pushNotification=window.plugins.pushNotification;
}
