if (typeof P=="undefined") {var P="prototype";}

function _PSU (numParts) {
	this.START_END_RND_CHARS="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890";
	this.HEX_ALPHA_DIGITS_PLUS_ZERO="ABCDEFX";
	if (numParts<3) {
		throw new Error("numParts must be at least 3.");
	}
	var numPartsHalved=numParts/2;
	if (numPartsHalved==Math.floor(numPartsHalved)) {
		throw new Error("numParts must be an odd number.");
	}
	this.numParts=numParts;
	this.padderChar=String.fromCharCode(0);
	this.randomStartCharsNum=2;
	this.randomEndCharsNum=3;
}

_PSU[P].getPartsFromString=function(str) {
	// first we bulk up the string by converting each char to its unicode code (padding those to 4 chars as needed); we do this
	// not to "encrypt" the string but so that when we break it into parts, there will be more parts of greater length, which
	// provides for more obfuscation, if for no other reason than that unicodes of each char will be split across parts (THAT
	// in turn is guaranteed by the fact that an even number of parts is not permitted; see the constructor):
	
	for (var i=0; i<PartedStringUtils.randomStartCharsNum; i++) {
		str=PartedStringUtils.START_END_RND_CHARS.charAt(Math.floor(Math.random()*PartedStringUtils.START_END_RND_CHARS.length))+str;
	}
	for (var i=0; i<PartedStringUtils.randomEndCharsNum; i++) {
		str+=PartedStringUtils.START_END_RND_CHARS.charAt(Math.floor(Math.random()*PartedStringUtils.START_END_RND_CHARS.length));
	}
	// first pad the string so its length is divisible by numParts (we strip the padder chars when we decode the parts):
	while (str.length/PartedStringUtils.numParts!=Math.floor(str.length/PartedStringUtils.numParts)) {
		str+=PartedStringUtils.padderChar;
	}
	var strLn=str.length;
	var unicodeStr="";
	var unicodeChar;
	for (var i=0; i<strLn; i++) {
		unicodeChar=""+str.charCodeAt(i);
		while (unicodeChar.length<4) {
			unicodeChar="0"+unicodeChar;
		}
		unicodeStr+=unicodeChar;
	}
	// reverse the string just for fun:
	unicodeStr=unicodeStr.reverse();
	
	var hexCharsLn=PartedStringUtils.HEX_ALPHA_DIGITS_PLUS_ZERO.length;
	
	while (unicodeStr.indexOf("0")>-1) {
		unicodeStr=unicodeStr.replace(/0/, PartedStringUtils.HEX_ALPHA_DIGITS_PLUS_ZERO.charAt(Math.floor(Math.random()*hexCharsLn)));		
	}
	unicodeStr=unicodeStr.replace(/X/g, "0");
	
	// now we break up the "unicoded" string:
	var parts=[PartedStringUtils.numParts];
	for (var i=0; i<PartedStringUtils.numParts; i++) {
		parts[i]=PartedStringUtils._getPart(unicodeStr, i);
	}
	return parts;
};

_PSU[P]._getPart=function(str, offset) {
	var part="";
	var strLn=str.length;
	for (var i=offset; i<strLn; i+=PartedStringUtils.numParts) {
		part+=str.charAt(i);
	}
	return part;
};

_PSU[P].getStringFromParts=function(parts) {
	var partsLn=parts.length;
	var partLn=parts[0].length; // all parts are the same length
	var unicodeStr="";
	for (var i=0; i<partLn; i++) {
		for (var p=0; p<partsLn; p++) {
			unicodeStr+=parts[p].charAt(i);
		}
	}
	// the string was reversed; un-reverse it:
	unicodeStr=unicodeStr.reverse();
	
	unicodeStr=unicodeStr.replace(/[A-F]/g, "0");

	unicodeStrLn=unicodeStr.length;
	var decoded="";
	var c;
	var unicodeSeq;
	for (var i=0; i<unicodeStrLn; i+=4) {
		unicodeSeq=unicodeStr.substring(i, i+4);
		while (unicodeSeq.charAt(0)=="0") {
			unicodeSeq=unicodeSeq.substring(1, unicodeSeq.length);
		}
		c=String.fromCharCode(parseInt(unicodeSeq));
		if (c!=PartedStringUtils.padderChar) {
			decoded+=c;
		}
	}
	return decoded.substring(PartedStringUtils.randomStartCharsNum, decoded.length-PartedStringUtils.randomEndCharsNum);
};

var PartedStringUtils=new _PSU(5);





function PartedString (baseStr) {
	this.parts=PartedStringUtils.getPartsFromString(baseStr);
}

PartedString[P].decode=function() {
	return PartedStringUtils.getStringFromParts(this.parts);
};

