
function _PinPad () {
	this.submitPinEntryWhenAllDigitsEntered=false; // True if for the currently visible PIN pad, PIN entry should be taken to be complete after six digits are entered
	this.currentPinSubmitFunction=function() {}; // The Function object to be called when user next chooses/enters a PIN (should never be set to null; should be set to function(){} for no-op
	this.lastUserEnteredPin=""; // Set to whatever the last PIN value was entered by the user (until cleared)
	// Following two values populated by CommMgr.establishSession()
	this.pinButtonTokensToDigitsMap=null;
	this.disallowedPinSequencesListArray=null;
	this.currentContainerId=null;
	this.pinPadElRefs=null;
	this.forceMobileAppLayout=true; // for now
}

var p=_PinPad.prototype;



p.updatePinDisplay=function(displayDigits) {
	var startOffsetLeftPx=((PinPad.forceMobileAppLayout || window.mobileApp)?1:-9);
	var currentInputLength=PinPad.lastUserEnteredPin.length;
	var noInput=(currentInputLength==0);
	
	if (PinPad.forceMobileAppLayout || window.mobileApp) {
		// we only need to show the instrux if the PIN was just cleared, and only need to hide if the PIN has length 1 (otherwise it was already hidden):
		if (noInput) {
			PinPad.showHideCurrentInstrux(true);
		}
		else if (currentInputLength==1) {
			PinPad.showHideCurrentInstrux(false);
		}
		
	}
	var endReached;
	var tokens=PinPad.pinButtonTokensToDigitsMap.keysArray();
	var digits=PinPad.pinButtonTokensToDigitsMap.valuesArray();

	
	var startIdx, endIdx;
	// define which slots we're changing; we used to do all of them, but on phones, this was a performance problem, so:
	if (noInput) {
		// if the length of the current input is 0, then the user just cleared the PIN; that's the only reason we'd be 
		// in this method with the length being zero.  So:
		startIdx=0;
		endIdx=5; // (note endIdx is inclusive).
	}
	else {
		// else they just clicked a numeral; we only process one numeral at a time, so it's not possible that we need to
		// update more than one numeral.  So:
		startIdx=(currentInputLength-1);
		endIdx=(currentInputLength-1); // (note endIdx is inclusive).
	}
	for (var i=startIdx; i<=endIdx; i++) {
		if (noInput) {
			PinPad.pinPadElRefs["circleOn"+i].hide();
			PinPad.pinPadElRefs["numeral"+i].hide();			
		}
		else {
			endReached=(i>=currentInputLength);
			if (!endReached) {
				if (!displayDigits) {
					PinPad.pinPadElRefs["circleOn"+i].show();
					PinPad.pinPadElRefs["numeral"+i].hide();
				}
				else {
					PinPad.pinPadElRefs["numeral"+i].html(digits[tokens.indexOf(PinPad.lastUserEnteredPin.charAt(i))]);
					PinPad.pinPadElRefs["numeral"+i].show();	
					PinPad.pinPadElRefs["circleOn"+i].hide();
				}
			}
			else {
				PinPad.pinPadElRefs["circleOn"+i].hide();
				PinPad.pinPadElRefs["numeral"+i].hide();
			}			
		}
	}

};



p.getPinPadHtml=function(displayDigits, submitButtonLabel) {
	var showSubmitButton=(submitButtonLabel!=null);
	var buttonsHtml=""
		// the display area above the actual pin buttons, where either entered digits or dots appear as user clicks on pin buttons:
		+"<form id=\"pinForm\" name=\"pinForm\" onsubmit=\"return false;\" action=\"#\">"
			+"<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" style=\"xbackground-color:#ffffff; margin:0 0 6px "+((PinPad.forceMobileAppLayout || window.mobileApp)?"23":"11")+"px;\"><tr><td>"
						+"<table border=\"0\" cellspacing=\"0\" cellpadding=\"0\" id=\"pinDisplayNumeralHolderTable\">"
							+"<tr>";
									var borderLeftWidthPx;
									var borderRightWidthPx;
									var widthOverrideCss; // used to pad our overall width to what it should be
									var numHolders=6;
									for (var i=0; i<numHolders; i++) {
										if (!(PinPad.forceMobileAppLayout || window.mobileApp)) {
											borderLeftWidthPx=(i==0?1:0);
											borderRightWidthPx=1;
											widthOverrideCss="";
										}
										else {
											borderLeftWidthPx=(i==0?2:0);
											borderRightWidthPx=(i==numHolders-1?2:0);
											widthOverrideCss=(i==numHolders-1?"width:41px; ":"");
										}

										buttonsHtml+="<td><div class=\"pinDisplayNumeralHolder\" style=\""+widthOverrideCss+"border-width:0;\" xstyle=\""+widthOverrideCss+"border-left-width:"+borderLeftWidthPx+"px; border-right-width:"+borderRightWidthPx+"px;\">"
											+"<img class=\"circleOff\" id=\"circleOff"+i+"\" src=\""+baseImagesUrlSkinned+"pinCircleOff"+((PinPad.forceMobileAppLayout || window.mobileApp)?"_mobile.svg":".png")+"\" />"
											+"<img class=\"circleOn\" id=\"circleOn"+i+"\" src=\""+baseImagesUrlSkinned+"pinCircleOn"+((PinPad.forceMobileAppLayout || window.mobileApp)?"_mobile.svg":".png")+"\" />"
											+"<div class=\"numeral\" id=\"numeral"+i+"\">&nbsp;</div>"
											+"</div></td>";
									}
							buttonsHtml+="</tr>"
						+"</table>"
					+"</td></tr>"
			+"</table>"
		+"</form>";
	var marginCss=((PinPad.forceMobileAppLayout || window.mobileApp)?"2px 14px 9px 4px":"4px 4px 4px 4px"); // more space for mobile, make it more difficult to thumb the wrong key
	
	// write the actual pin-digit keys:
	var tokens=PinPad.pinButtonTokensToDigitsMap.keysArray();
	var digits=PinPad.pinButtonTokensToDigitsMap.valuesArray();
	var digit, token, leftPx, onmousedown;
	var idx;
	var clearBtnSlot;
	var submitBtnSlot;
	var emptySlot;
	var numeralSlot;
	var show;
	for (var i=1; i<=12; i++) {
		idx=i;
		clearBtnSlot=false;
		submitBtnSlot=false;
		emptySlot=false;
		numeralSlot=true;

		// all last-row slots are special cases:
		if (i==10) {
			clearBtnSlot=true;
			numeralSlot=false;
		}
		else if (i==11) {
			idx=10;
		}
		else if (i==12) {
			numeralSlot=false;
			if (showSubmitButton) {
				submitBtnSlot=true;
			}
			else {
				emptySlot=true;
			}
		}
		digit=parseInt(digits[idx-1]);
		token=tokens[idx-1];
		leftPx=1-(digit*70);
		if (PinPad.forceMobileAppLayout || window.mobileApp) {
			leftPx+=21;
		}
		
		onmousedown="PinPad.pinButtonClick('"+token+"', "+displayDigits+"); ";
		if (PinPad.forceMobileAppLayout || window.mobileApp) {
			onmousedown+="PinPad.mobileTapFeedback('"+idx+"', 'show');";
			//onmousedown+="ClickHandlers.cancelEvent(event);";
			onmousedown+="setTimeout(function(){PinPad.mobileTapFeedback('"+idx+"', 'hide');}, 125); ";
		}
		//onmousedown+="return false;"; // fixes issues on mobile (underlying image getting highlighted)
		
		var onMouseDownAttr=(!(window.mobileApp) || window.localMobileAppDebug?"onmousedown":"ontouchstart");
		
		if (numeralSlot) {
			// NO ONCLICKS HERE!!!  For some reason, they SERIOUSLY degraded performance on mobile, especially iOS; so we use mousedown/touchstart.
			buttonsHtml+="<div "+onMouseDownAttr+"=\""+onmousedown+"\" class=\"pinButtonHolder\" style=\"margin:"+marginCss+";\">"
			if (PinPad.forceMobileAppLayout || window.mobileApp) {
				buttonsHtml+="<img "
					+"src=\""+baseImagesUrlSkinned+"pinBgActive_mobile.svg\" "
					+"onerror=\"UiMgr.doSvgMobileFallback(this)\" "
					+"id=\"keyActiveBg"+idx+"\" class=\"pinDigitsImgActiveBg\" />";			
			}
			
			buttonsHtml+="<div "
				+"onerror=\"UiMgr.doSvgMobileFallback(this)\" onclick=\"return false;\" onfocus=\"return false;\" onselectstart=\"return false;\" onmousedown=\"return false;\" ontouchstart=\"return false;\" "
				+"id=\"key"+idx+"\" class=\"pinDigitsImg\" style=\"background-image:url("+baseImagesUrlSkinned+"pinNumbersWithZero"+((PinPad.forceMobileAppLayout || window.mobileApp)?"_mobile.svg":".png")+"); left:"+leftPx+"px;\">&nbsp;</div></div>";			
		}
		else if (emptySlot) {
			buttonsHtml+="<div class=\"pinButtonHolder emptySlot\" style=\"margin:"+marginCss+";\">&nbsp;</div>"			
		}
		else if (clearBtnSlot) {
			buttonsHtml+="<div class=\"pinButtonHolder nonNumeralSlot\" style=\"margin:"+marginCss+";\">"
				+"<button id=\"pinPadClear\" class=\"genericButton pinClearBtn\"  onclick=\"PinPad.pinButtonClick('clear', "+displayDigits+")\">clear</button>"
				+"</div>"			
		}
		else if (submitBtnSlot) {
			buttonsHtml+="<div class=\"pinButtonHolder nonNumeralSlot\" style=\"margin:"+marginCss+";\">"
				+"<button id=\"pinPadSubmit\" class=\"genericButton pinSubmitBtn\"  onclick=\"PinPad.currentPinSubmitFunction()\">"+submitButtonLabel+"</button>"
				+"</div>"			
		}
	}
	return buttonsHtml;
};


p.mobileTapFeedback=function(idx, showOrHide) {
	setTimeout(function(){$("#keyActiveBg"+idx)[showOrHide]();}, 0);
};


/**
 * @param displayDigits boolean, display the digits entered, or just dots
 * @param containerId the id of the container into which to write the pad
 * @param submitButtonLabel the label for the submit button; null means don't show a button (in which case submitOnAllDigitsEntered must be true)
 * @param submitOnAllDigitsEnteredboolean, true if we should submit the digits after the user has entered all of them (6 of them); if this is true, submitButtonLabel must be null
 * @param submitFunction a Function (not a function name; an actual Function object) to be called when the PIN is submitted (regardless of how it's submitted, which is determined by the values of submitButtonLabel and submitOnAllDigitsEntered
 * @param instrux Text to appear over the "textbox" (for mobile, and only when no digits have been entered), or preceding the PIN UI (for desktop)
 */
p.showInPagePinPad=function(displayDigits, containerId, submitButtonLabel, submitOnAllDigitsEntered, submitFunction, instrux) {
	if (typeof instrux=="undefined") {
		throw new Error("PinPad.showInPagePinPad() requires six args; none are optional.");
	}
	if (submitButtonLabel==null && !submitOnAllDigitsEntered) {
		throw new Error("No button label specified but submitOnAllDigitsEntered is false.");
	}
	if (submitButtonLabel!=null && submitOnAllDigitsEntered) {
		throw new Error("Button label specified but submitOnAllDigitsEntered is true.");
	}
	if (submitFunction==null) {
		throw new Error("submitFunction cannot be null.");
	}
	PinPad.lastUserEnteredPin="";
	PinPad.hideAllInPagePinPads();
	PinPad.submitPinEntryWhenAllDigitsEntered=submitOnAllDigitsEntered;
	PinPad.currentPinSubmitFunction=submitFunction;
	PinPad.currentContainerId=containerId;
	var html="<div id=\""+containerId+"_instruxOverlay\" class=\"instruxOverlay\">"+instrux+"</div>";
	html+="<div id=\""+containerId+"_content\" style=\""+(!mobileApp?"margin-left:14px;":"")+"\" class=\"content\">"+PinPad.getPinPadHtml(displayDigits, submitButtonLabel)+"</div>";
	//.pageContentWrapper.fullBleed.inPagePinPadHolder .instruxOverlay {position:absolute; top:10px; left: 10px; width:200px; z-index:20;}


	$("#"+containerId).html(html);
	// Set up refs to DOM els we update frequently, to help with performance:
	PinPad.pinPadElRefs=[];
	PinPad.pinPadElRefs.instruxOverlay=$("#"+containerId+"_instruxOverlay");
	for (var i=0; i<6; i++) {
		PinPad.pinPadElRefs["circleOn"+i]=$(".pinDisplayNumeralHolder #circleOn"+i); // note: no need to ref circleOff els; we never hide them
		PinPad.pinPadElRefs["numeral"+i]=$(".pinDisplayNumeralHolder #numeral"+i);
	}
	PinPad.showHideCurrentInstrux(true);
	$(".genericButton").button(); // force jquery-ui to style the button we just wrote; it already does that, but this is a new button in the DOM, so we have to tell it explicitly to do this

};


p.showHideCurrentInstrux=function(showBoolean) {
	if (!window.mobileApp) { // we don't show/hide if not mobile --
		return;
	}
	PinPad.pinPadElRefs.instruxOverlay[showBoolean?"show":"hide"]();	
	
};



p.hideAllInPagePinPads=function() {
	$(".inPagePinPadHolder").html("&nbsp;");
	PinPad.submitPinEntryWhenAllDigitsEntered=false;
	PinPad.currentContainerId=null;
};





p.validateNewLastEnteredPin=function() {
	return PinPad.validatePin("generic", PinPad.lastUserEnteredPin);
};

p.validatePin=function(typeCode, pinVal) { // typeCode one of: generic, auth; tempKeyAuth; creatingUserNewPin; secondUserNewPin; reset
	var alertCallbackFn;
	var newChosenPin;
	if (typeCode=="generic") {
		alertCallbackFn=null;
		newChosenPin=true;
	}
	else if (typeCode=="auth") {
		alertCallbackFn=PinPad.openPinDialogForAuth;
		newChosenPin=false;
	}
	else if (typeCode=="creatingUserNewPin") {
		alertCallbackFn=null;
		newChosenPin=true;
	}
	else if (typeCode=="secondUserNewPin") {
		alertCallbackFn=null;
		newChosenPin=true;
	}
	else if (typeCode=="tempKeyAuth") {
		alertCallbackFn=PinPad.openPinDialogForTempKeyAuth;
		newChosenPin=false;
	}
	else {
		throw new Error("typeCode "+typeCode+" not recognized");
	}
	
	if (pinVal.length==0) {
		var msg=(newChosenPin
			?"You haven't chosen a PIN. Please choose a six-digit PIN and enter it. You will use this PIN every time you sign in."
			:"You didn't enter a PIN.  Please try again."
		);
		PinPad.lastUserEnteredPin="";
		PinPad.clearPin();
		DialogMgr.alert(msg, alertCallbackFn);
		return false;
	}
	if (pinVal.length!=6) {
		var msg=(newChosenPin
			?"The PIN you chose wasn't six digits long. Please try again. You will use this PIN every time you sign in."
			:"The PIN you entered wasn't six digits long.  Please try again."
		);
		PinPad.lastUserEnteredPin="";
		PinPad.clearPin();

		DialogMgr.alert(msg, "OK", alertCallbackFn);

		return false;
	}
	if (newChosenPin) {
		// we're not authenticating the user; the user is choosing a new pin (or a replacement one); therefore we also need to do some other checks:
		var goodPin=true;
		// we use Map as a Set here (ignoring the values; we only care about the keys; we're counting instances of different numerals):
		var numeralsSet=new Map();
		for (var i=0; i<pinVal.length; i++) {
			numeralsSet.put(pinVal.charAt(i), "");
		}
		if (numeralsSet.keysArray().length<2) {
			goodPin=false;
		}
		else {
			// else we've got enough check our list of disallowed substrings:
			for (var i=0; i<PinPad.disallowedPinSequencesListArray.length; i++) {
				if (pinVal.indexOf(PinPad.disallowedPinSequencesListArray[i])>-1) {
					goodPin=false;
					break;
				}
			}
		}
		if (!goodPin) {
			PinPad.lastUserEnteredPin="";
			PinPad.clearPin();
			DialogMgr.alert("The PIN you chose is too predictable; please try again.  Your PIN must be difficult for anyone to guess.", "OK", alertCallbackFn);
			return false;			
		}
	}
	return true;
};


p.clearPin=function(displayDigits) {
	PinPad.lastUserEnteredPin="";
	PinPad.updatePinDisplay(displayDigits);
};



p.pinButtonClick=function(code, displayDigits) {
	var updateDisplay=false;
	if (code=="clear") {
		PinPad.lastUserEnteredPin=""; // think this is redundant; fk it
		PinPad.clearPin(displayDigits);
		return;
	}
	else {
		var currVal=PinPad.lastUserEnteredPin;
		if (currVal.length<6) {
			PinPad.lastUserEnteredPin+=code;
			if (PinPad.lastUserEnteredPin.length==6 && PinPad.submitPinEntryWhenAllDigitsEntered) {
				var fn=PinPad.currentPinSubmitFunction;
				if (fn==null) {
					fn=function(){};
				}
				setTimeout(function(){fn();}, 100);
			}
			updateDisplay=true;
		}
		else {
			// else nothing; this alert was superfluous - if user types beyond 6 digits, they just won't be able to type anymore --
			// DialogMgr.alert("Too many digits! The maximum is 6.");
		}
	}
	if (updateDisplay) {
		PinPad.updatePinDisplay(displayDigits);
	}
};

var PinPad=new _PinPad();
