

function _ScreenViewStack () {	
	/*
	 * 	Following is an array representing the history of the user's screen views.  Values are codes reflecting 
	 * screens (dialogs don't count, including footer-link content on desktop UI, also including dialogs which 
	 * look like "real" pages on mobile).  Codes are either an "authPage" code (e.g. "authEnterPin"), or one of 
	 * these: "settings" or "messengerUi:id", where "id" is the clientSessionTunnelId which was being viewed in the
	 * messenger UI. It
	 * is usually set by UiMgr.showAuthPage() (in the case of all "authPage" pages); for settings/messengerUi,
	 * it is set by ClickHandlers or enterTunnel/exitTunnel functions in MessengerMgr.  In some cases, we use it
	 * for reference (to determine what the last page viewed was).  Occasionally we clear it completely (so as not
	 * to leave any history of user activity).  We never remove ids from the stack; if we "go back" to a previous
	 * screen, we simply  add that to the history.  E.g., if we are on screen A, go to screen B, then want to
	 * return to screen A (because for example screen B is the settings page, and after changing settings, we want
	 * to return user to previous page), we simply add A to the end of the stack when we "return", resulting in
	 * an array whose last three elements are A, B, A.
	 * 
	 */
	this._stack=[]; 
	this.MAX_STACK_SIZE=5;
}

var p=_ScreenViewStack.prototype;



/** Add to the history of screens viewed.  This method also cleans up the array that backs the stack, keeping it to
 * a length of 5 or less (we don't need more history than that).
 * @param id An "authPage" code, "help" (mobile only), or "messengerUi" if the screen is the messaging UI
 * @param tunnelId Our ID for the tunnel currently being viewed in the messaging UI (null -- or not passed -- in all cases when id !=messengerUi)
 */
p.add=function(id, tunnelId) {
	//console.info("in ScreenViewStack.add(), id="+id);
	if (typeof tunnelId=="undefined") {
		tunnelId=null;
	}
	if (id=="messengerUi" && tunnelId==null) {
		throw new Error("id is messengerUi but no tunnelId specified.");
	}
	if (id!="messengerUi" && tunnelId!=null) {
		throw new Error("id is not messengerUi but tunnelId was specified.");
	}
	ScreenViewStack._stack.push(id+(id!="messengerUi"?"":":"+tunnelId));
	if (ScreenViewStack._stack.length>(ScreenViewStack.MAX_STACK_SIZE)) {
		ScreenViewStack._stack.shift();
	}
	//console.log("DEBUG: current _stack="+ScreenViewStack._stack);
};


p.clear=function() {
	ScreenViewStack._stack=[];
};

p.getPreviousScreenId=function() {
	if (ScreenViewStack._stack.length<=1) {
		return null;
	}
	return ScreenViewStack._stack[ScreenViewStack._stack.length-2];
};



p.getCurrentScreenId=function() {
	if (ScreenViewStack._stack.length==0) {
		return null;
	}
	return ScreenViewStack._stack[ScreenViewStack._stack.length-1];
};


p.isMessagingUiActive=function() {
	//console.info("ScreenViewStack.getCurrentScreenId()="+ScreenViewStack.getCurrentScreenId());
	return ScreenViewStack.getCurrentScreenId().startsWith("messengerUi:");
};









var ScreenViewStack=new _ScreenViewStack();

