
function _TunnelMgr () {
	this.REQUIRED_TUNNEL_LIST_ENTER_TUNNEL_ITEM_DRAG_PX=120; // number of px user has to drag an item to the left to trigger the item's click() (which takes them into the tunnel)
	this.REQUIRED_TUNNEL_LIST_FOR_DELETE_REVEAL_ITEM_DRAG_PX=130; // number of px user has to drag an item to the right to lock in "delete mode" for this item
	this._tunnels=new Map(); // Map of local tunnel ids to Tunnel objects
	this.ignoreItemClicks=false; // set to true for a few millis as a hack to get around issues when user drags item, but not enough to trigger click (but the actual click event still fires)
	this.currentClientSessionTunnelId=null; // our local, session-specific, ID for the current tunnel, if any
	this._currentClientDetails=[]; // JS object: name/val pairs such as fullname, address, email (see EnterTunnelRequestHandler for details); equals [] (empty object) unless logged-in acct is pro originator acct
	this.addDeleteMode="add"; // values: "add" or "delete"
}



var p=_TunnelMgr.prototype;


p.reset=function() {
	this._tunnels.clear();
	this.currentClientSessionTunnelId=null;
	this._currentClientDetails=[];
	this.ignoreItemClicks=false;
};

p.getTunnelsCount=function() {
	return this._tunnels.size();	
};

p.getTunnelIdsToTunnelsMap=function() {
	return this._tunnels;	
};

p.getFirstTunnelLocalId=function() {
	return this._tunnels.keysArray()[0];	
};



/**
 * Creates and adds a Tunnel object and returns it
 */
p.addTunnel=function(localId, name, currentUserName, confidantName, hasUnreadMessages, confidantOnline, timeSpent) {
	var tunnel=new Tunnel(localId, name, currentUserName, confidantName, hasUnreadMessages, confidantOnline, timeSpent);
	this._tunnels.put(localId, tunnel);
	return tunnel;
};


/**
 * @param localId Our local ID for this tunnel
 * @param attrName E.g. hasUnreadMessages if we're changing the boolean attr indicating if there are unread messages in this tunnel
 * @param attrValue a String or boolean value for the specified attr (converted to booleans for hasUnreadMessages and confidantOnline, converted to String otherwise)
 */
p.changeTunnelAttribute=function(localId, attrName, attrValue) {
	if (attrName=="localId") {
		throw new Error("Can't change a Tunnel's localId");
	}
	if (attrName=="hasUnreadMessages" || attrName=="confidantOnline") {
		if ((""+attrValue).toLowerCase()=="true") {
			attrValue=true;
		}
		else {
			attrValue=false;
		}
	}
	else {
		attrValue=""+attrValue;
	}
	this._tunnels.get(localId)[attrName]=attrValue;
};



/**
 * @param localId Our local ID for this tunnel
 * @param attrName E.g. hasUnreadMessages if we're changing the boolean attr indicating if there are unread messages in this tunnel
 * @param attrValue a String or boolean value for the specified attr (converted to booleans for hasUnreadMessages and confidantOnline, converted to String otherwise)
 */
p.getTunnelAttribute=function(localId, attrName) {
	var  tunnel=this._tunnels.get(localId);
	if (typeof tunnel=="undefined" || tunnel==null) {
		return null;
	}
	return tunnel[attrName];
};

p.getCurrentTunnelAttribute=function(attrName) {
	if (this.currentClientSessionTunnelId==null) {
		return null;
	}
	return TunnelMgr.getTunnelAttribute(this.currentClientSessionTunnelId, attrName);
};

/**
 * See EnterTunnelRequestHandler for possible values for name param
 */
p.getCurrentClientDetail=function(name) {
	if (this.currentClientSessionTunnelId==null) {
		return "";
	}
	var val=TunnelMgr._currentClientDetails[name];
	return (typeof val=="undefined" || val==null?"":val);
};



/*
 * Returns the current tunnel's confidantName, or "Your Confidant" if confidantName is __UNREG_SECOND_USER__
 */
p.getParsedCurrentConfidantName=function() {
	var confName=TunnelMgr.getUnparsedCurrentConfidantName();
	if (confName==UiMgr.unregisteredSecondUserNicknameToken) {
		return UiMgr.unregisteredConfidantIdentifier;
	}
};


/*
 * Returns the current tunnel's confidantName (not parsed, so __UNREG_SECOND_USER__ if that's what it is)
 */
p.getUnparsedCurrentConfidantName=function() {
	return TunnelMgr.getCurrentTunnelAttribute("confidantName");
};







/**
 * Removes the Tunnel based on its local id
 * @param localId the tunnel's local id
 */
p.removeTunnel=function(localId) {
	this._tunnels.remove(localId);
};


/**
 * refreshes the _tunnels Map with new data from JSON passed to us via AJAX (we don't do adding/updating/deleting of Tunnels here;
 * this is brute-force: just re-create the whole Map and then switch it out for the old one).
 */
p.refresh=function(tunnelsData) {
	var newMap=new Map();
	var tunnelData;
	var confidantName;
	for (var localId in tunnelsData) {
		tunnelData=tunnelsData[localId];
		confidantName=tunnelData.confidantName;
		if (confidantName==UiMgr.unregisteredSecondUserNicknameToken) {
			confidantName=UiMgr.unregisteredConfidantIdentifier;
		}
		newMap.put(tunnelData.localId, new Tunnel(tunnelData.localId, tunnelData.name, tunnelData.currentUserName, confidantName, tunnelData.hasUnreadMessages, tunnelData.confidantOnline, tunnelData.timeSpent));
	}
	this._tunnels=newMap;
	newMap=null;
};







/**
 * The user just finished a drag event of a tunnel list item; determine what to do (minimally, we'll return the element to its pre-drag x position):
 */
p.tunnelListItemDragStop=function(event, ui) {
	var currentLeft=ui.position.left; // we could get the top too, but we don't care about it, because it hasn't changed
	var draggedElement=ui.helper; // a jQuery representing the dragged element
	// move the dragged el's position back to where it was:
	

	if (currentLeft<((TunnelMgr.REQUIRED_TUNNEL_LIST_ENTER_TUNNEL_ITEM_DRAG_PX)*-1)) {
		draggedElement.css("left", "0px");
		// a drag to the left, far enough to reveal "enter this tunnel":
		//console.info("enter-tunnel drag");
		TunnelMgr.ignoreItemClicks=false;
		setTimeout(function(){draggedElement.click();}, 0);
	}
	else if (currentLeft>TunnelMgr.REQUIRED_TUNNEL_LIST_FOR_DELETE_REVEAL_ITEM_DRAG_PX) {
		// a drag to the right, far enough to reveal the delete-related items.  In this case, we actually do nothing; the next
		// step is either for the user to click the revealed "delete" button, or for them to slide the thing back.
		draggedElement.css("left", ""+(TunnelMgr.REQUIRED_TUNNEL_LIST_FOR_DELETE_REVEAL_ITEM_DRAG_PX-3)+"px");
		//console.info("delete drag");
		TunnelMgr.ignoreItemClicks=true;
		setTimeout(function(){TunnelMgr.ignoreItemClicks=false;}, 100);
	}
	else {
		draggedElement.css("left", "0px");
		//console.info("no-op drag");
		// the drag operation that didn't go far enough to the left will still fire the click() on the item if the user's mouse/touch was over the item on release, so:
		TunnelMgr.ignoreItemClicks=true;
		setTimeout(function(){TunnelMgr.ignoreItemClicks=false;}, 100);
	}
	
};





/**
 * Leave the current tunnel (log this account's user of this tunnel out of it and return to the tunnels-list page)
 */
p.exitTunnel=function(forceShowTunnelList) {
	if (typeof forceShowTunnelList=="undefined") {
		forceShowTunnelList=false;
	}

	if (mobileApp) {
		try {
			VirtualKeyboard.setKbVisible(false);
		}
		catch (e) {}
	}
	
	if (!window.mobileApp) {
		UiMgr.switchFooterSocialLinksVisibility(false);
	}
	MessengerMgr.doUpdateChecks=false;
	clearTimeout(MessengerMgr.updateCheckTimeout);
	$("#transparentMediaFileUploadFieldWrapper").html("&nbsp;");
	if (TunnelMgr.currentClientSessionTunnelId!=null) {
		CommMgr.initiateAjaxRequest(
				CommMgr.getControlServer(), 
				UiMgr.handlerKeys.LEAVE_TUNNEL, 
				{localId:TunnelMgr.currentClientSessionTunnelId}, 
				true, 
				function() {
					TunnelMgr.currentClientSessionTunnelId=null;
					AccountUpdateRetriever.requestUpdate(true);
				},
				function() {},
				30000
			);		
	}
	if (forceShowTunnelList) {
		UiMgr.showAuthPage("authTunnelList");
	}
	$("#correspondentStatusMessage").hide();
};

p.switchAddDeleteMode=function(mode) {
	if (typeof mode=="undefined" || mode==null || (!mode=="add" && !mode=="delete")) {
		throw new Error("Missing or invalid mode (must be 'add' or 'delete')");
	}
	TunnelMgr.addDeleteMode=mode;
	$("#tunnelListWrapper .tunnelListItem .draggableOverlay .desktopArrowOrBomb #arrow").css("display", mode=="add"?"block":"none");
};


/*
 * "Client" users of professional sites (e.g. Tunnel T) only have one tunnel; calling this method opens that tunnel's messaging UI (and is generally
 * called when the call would otherwise be to the account's list of tunnels; client-users never see their list of tunnels because they always have
 * exactly one tunnel.
 */
p.singleTunnelAccountEnterTunnel=function(isFirstTunnelVisit) {
	isFirstTunnelVisit=(typeof isFirstTunnelVisit=="undefined"?false:isFirstTunnelVisit);
	AccountUpdateRetriever.start();
	setTimeout(function() {
		DialogMgr.openBlockingModalDialog("One moment...");
		setTimeout(function() {
			DialogMgr.closeBlockingModalDialog();
			//alert("TunnelMgr._tunnels.isEmpty()="+TunnelMgr._tunnels.isEmpty());
			if (!TunnelMgr._tunnels.isEmpty()) {
				//alert("TunnelMgr._tunnels.keysArray()[0]="+TunnelMgr._tunnels.keysArray()[0]);
				var msg=null;
				if (isFirstTunnelVisit) {
					if (!mobileApp) {
						msg="Congratulations, you have created an account. This is where you'll have an ongoing, secure conversation with your "+UiMgr.skinnedSiteData.originatorName.toLowerCase()+".<br/><br/><b>You must remember the image key and PIN that you chose! You will need them every time you log in, and your "+UiMgr.skinnedSiteData.originatorName.toLowerCase()+" can't retrieve them for you if you lose them.</b><br/><br/>If you have any issues using "+UiMgr.skinnedSiteData.siteName+", please contact your "+UiMgr.skinnedSiteData.originatorName.toLowerCase()+"."
					}
					else {
						msg="Congratulations, you have created an account. This is where you'll have an ongoing, secure conversation with your "+UiMgr.skinnedSiteData.originatorName.toLowerCase()+".<br/><br/><b>You must remember the PIN that you just chose! You will need it every time you log in, and your "+UiMgr.skinnedSiteData.originatorName.toLowerCase()+" can't retrieve it for you if you lose it.</b><br/><br/>If you have any issues using "+UiMgr.skinnedSiteData.siteName+", please contact your "+UiMgr.skinnedSiteData.originatorName.toLowerCase()+"."
					}
				}
				TunnelMgr.enterTunnel(TunnelMgr._tunnels.keysArray()[0], msg);
			}
			else {
				DialogMgr.alert("Either you don't have an internet connection, or the user who created your tunnel has deleted it; please contact that user for details.");
			}
			AccountUpdateRetriever.stop();
		}, 1000);
	}, 50);
};


p.enterTunnel=function(clientSessionTunnelId, postEntryMessage) {
	MessengerMgr.switchClientInfoEditMode(false);

	postEntryMessage=(typeof postEntryMessage=="undefined"?null:postEntryMessage);
	AccountUpdateRetriever.requestUpdate(true);
	//DialogMgr.openBlockingModalDialog("Signing you into \""+TunnelMgr.getTunnelAttribute(clientSessionTunnelId, "name")+"...");
	DialogMgr.openBlockingModalDialog("One moment...");
	if (!window.mobileApp) {
		UiMgr.switchFooterSocialLinksVisibility(true);
		if (professionalSite && UiMgr.currentAcctIsProfessionalClient) { // hide the tabs for everyone but pro-account originating accts
			$("#messengerUiTabs").hide();
		}
	}
	

	setTimeout(function(){
		try {
			
			var nonceSuccess=false;
			var dataMap={localId:clientSessionTunnelId};
			try {
				// The handler we're about to talk to requires that we get a nonce from the server, generate our 
				// own nonce, and hash them with the clientSessionTunnelId, then set those values in the form, so:
				//CommMgr.addNonceAndHashToAjaxDataObject(UiMgr.handlerKeys.ENTER_TUNNEL, dataMap, clientSessionTunnelId);
				nonceSuccess=true;
			}
			catch (e) {
				// Didn't work; user probably lost internet connection (or has an extremely slow one):
				console.error(e);
				DialogMgr.closeBlockingModalDialog();
				DialogMgr.alert("It looks like you don't have an internet connection.  Please wait a bit, then try again.");
				return;
				
			}
			if (nonceSuccess) {
				CommMgr.initiateAjaxRequest(
						CommMgr.getControlServer(), 
						UiMgr.handlerKeys.ENTER_TUNNEL, 
						dataMap, 
						true, 
						function(response){
							DialogMgr.closeBlockingModalDialog();
							var sendButton=$("#messengerSendButton");
							var sendOnEnterNote=$("#enterToSendNote");
							if (""+UiMgr.acctPrefs.sendMessageOnEnterAcctPref=="true") {
								sendButton.hide();
								sendOnEnterNote.show();
							}
							else {
								sendButton.show();
								sendOnEnterNote.hide();
							}
							// start the control-server status check; we do a timeout, not an interval,
							// because after each response from the update request, we do another
							// timeout (with an interval, requests could "pile up"):
							MessengerMgr.doUpdateChecks=true;
							clearTimeout(MessengerMgr.updateCheckTimeout);
							MessengerMgr.updateCheckTimeout=setTimeout(MessengerMgr.doUpdateCheck, MessengerMgr.standardUpdateCheckIntervalMillis);
							TunnelMgr.currentClientSessionTunnelId=clientSessionTunnelId;
							TunnelMgr._currentClientDetails=response.clientInfo;

								
							// set the tunnel-header fields:
							var tunnelName=TunnelMgr.getTunnelAttribute(TunnelMgr.currentClientSessionTunnelId, "name");
							var confidantName;
							if (professionalSite && !UiMgr.currentAcctIsProfessionalClient) {
								confidantName=tunnelName; // this is in fact the confidant's name too, regardless of whether they've reciproated
							}
							else {
								confidantName=TunnelMgr.getTunnelAttribute(TunnelMgr.currentClientSessionTunnelId, "confidantName");								
							}
							if (confidantName==UiMgr.unregisteredSecondUserNicknameToken) {
								confidantName=UiMgr.unregisteredConfidantIdentifier;
							}
							var showConfName=(confidantName!=UiMgr.unregisteredConfidantIdentifier);

							if (window.mobileApp) {
								$("#conversationUiHeader #tunnelInfo #tunnelInfoText #tunnelName").html(tunnelName);
								$("#conversationUiHeader #tunnelInfo #tunnelInfoText #confidantName").html(confidantName);
								$("#conversationUiHeader #tunnelInfo #tunnelInfoText #confidantNameInParens")[showConfName?"show":"hide"]();

							}
							else {
								// for client-users of professional sites, don't show their tunnel name at the top; it's redundant:
								if (!UiMgr.currentAcctIsProfessionalClient) {
									$("#mainUiUpperTable #tunnelInfoText #tunnelName").html(tunnelName);									
								}
								//$("#mainUiUpperTable #tunnelInfoText #confidantName").html(confidantName);
								//$("#mainUiUpperTable #tunnelInfoText #confidantNameInParens")[showConfName?"show":"hide"]();
								$("#mainUiUpperTable #tunnelInfoText #confidantNameInParens").hide();
								$("#correspondentStatusMessage").show();
								var fullname=TunnelMgr.getCurrentClientDetail("fullname");								
								var email=TunnelMgr.getCurrentClientDetail("email");
								var address=TunnelMgr.getCurrentClientDetail("address");								
								var telephone=TunnelMgr.getCurrentClientDetail("telephone");								
								var cityStatePostal=TunnelMgr.getCurrentClientDetail("cityStatePostal");								
								var mobileTelephone=TunnelMgr.getCurrentClientDetail("mobileTelephone");								
								var workTelephone=TunnelMgr.getCurrentClientDetail("workTelephone");								
								var generalNotes=TunnelMgr.getCurrentClientDetail("generalNotes");								
								var medications=TunnelMgr.getCurrentClientDetail("medications");								
								$("#proClientInfo #fullname #fullnameInput").val(fullname.length>0?fullname:"(name not entered)");
								$("#proClientInfo #email #emailInput").val(email.length>0?email:"(email not entered)");
								//$("#proClientInfo #email a").attr("href", email.length>0?"mailto:"+email:"javascript:DialogMgr.alert('You have not entered an email address.')");
								$("#proClientInfo #address #addressInput").val(address.length>0?address:"(address not entered)");
								$("#proClientInfo #telephone #telephoneInput").val(telephone.length>0?telephone:"(not entered)");
								$("#proClientInfo #cityStatePostal #cityStatePostalInput").val(cityStatePostal.length>0?cityStatePostal:"(city/state/postal not entered)");
								$("#proClientInfo #mobileTelephone #mobileTelephoneInput").val(mobileTelephone.length>0?mobileTelephone:"(not entered)");
								$("#proClientInfo #workTelephone #workTelephoneInput").val(workTelephone.length>0?workTelephone:"(not entered)");
								$("#proClientInfo #generalNotesInput").val(generalNotes.length>0?generalNotes:"(no notes entered)");
								if (window.showSecondClientInfoTextBlock) {
									$("#proClientInfo #medicationsInput").val(medications.length>0?medications:"(no medication info entered)");									
								}
								else {
									// don't hide() it; we still set/get its values (to be minimally intrusive), but user never sees it:
									$("#proClientInfo #medicationsHeader").css("visibility","hidden");																		
									$("#proClientInfo #medicationsInput").css("visibility","hidden");																		
								}
								

							}
							
							// start the checks that will log us out if we lose our connection:
							// CommMgr.startLastResponseTimeChecks(); // needed anymore?
							MessengerMgr.showMessengerUi();
							
							
							ScreenViewStack.add("messengerUi", clientSessionTunnelId);
							// clear previous instances of this form if we've written them; otherwwise we may end up with duplicte forms on the
							// same page, which will be a nightmare bug:
							UiMgr.clearAllTransparentUploadFileForms();
							if (UiMgr.transparentFormElHtml==null) { // first use -- so populate this value:
								UiMgr.transparentFormElHtml=$("#transparentFileUploadFormTemplate").html().replace(/\_FORMTEMPLATE\_/g, "");
							}
							$("#transparentMediaFileUploadFieldWrapper").html(UiMgr.transparentFormElHtml);					

							UiMgr.unalertedPageExitOkay=false;
							UiMgr.unregisteredConfidantAlertAsNeeded();
							var confidantName;
							//alert(1)
							if (professionalSite && !UiMgr.currentAcctIsProfessionalClient) {
								confidantName=TunnelMgr.getTunnelAttribute(TunnelMgr.currentClientSessionTunnelId, "name"); // use the tunnel name here, which is the confidant's name, even if they haven't reciprocated yet						
								//alert(2+confidantName);
							}
							else {
								confidantName=TunnelMgr.getTunnelAttribute(TunnelMgr.currentClientSessionTunnelId, "confidantName");
								//alert(3+confidantName)
							}
							
							if (confidantName==UiMgr.unregisteredSecondUserNicknameToken) {
								confidantName=UiMgr.unregisteredConfidantIdentifier;
							}

							MessengerMgr.setNamesForStatusMsgs(TunnelMgr.getCurrentTunnelAttribute("currentUserName"), confidantName);
							if (postEntryMessage!=null) {
								setTimeout(function() {DialogMgr.alert(postEntryMessage);}, 100);
							}

						}, 
						function(){
							DialogMgr.closeBlockingModalDialog();
							DialogMgr.alert("Sorry, couldn't log you in.  Please make sure you have an internet connection and try again.");
						}, 
						30000);
				
			}
			if (!window.mobileApp) {
				// CKEditor advertises itself (via mouseover, in an ugly way), so:
				$("#cke_msgCompose_arialbl").html("&nbsp;");
				$(".cke_wysiwyg_div").attr("title", "");
				$(".cke_wysiwyg_div").attr("aria-label", "");
			}
		}
		catch (e) {
			DialogMgr.closeBlockingModalDialog();
			DialogMgr.alert("Sorry, couldn't log you in.  Please make sure you have an internet connection and try again.");
		}
		
		
	}, 100);
};






/**
 * @param The local id (persistent only through this session, issued by the server) of the Tunnel (a Base64 string preceded by "x")
 * @param The tunnel name
 * @param The logged-in user's name (in this tunnel)
 * @param The confidant user's name (or null if nonexistent)
 * @param hasUnreadMessages True if there are messages in the tunnel which this user has never been in the tunnel to see
 * @param confidantOnline true if the confidant is online right now
 * @param timeSpent The amount of time spent (h:mm) by the professional, originating account (or "0:00" for all others, not expected to be displayed)
 */
function Tunnel (localId, name, currentUserName, confidantName, hasUnreadMessages, confidantOnline, timeSpent) {
	this.localId=localId;
	this.name=name;
	this.currentUserName=currentUserName;
	this.confidantName=confidantName;
	this.hasUnreadMessages=hasUnreadMessages;
	this.confidantOnline=confidantOnline;
	this.timeSpent=timeSpent;
}

Tunnel[P].toString=function() {
	return "Tunnel:["
	+"localId="+this.localId+","
	+"name="+this.name+","
	+"currentUserName="+this.currentUserName+","
	+"confidantName="+this.confidantName+","
	+"hasUnreadMessages="+this.hasUnreadMessages+","
	+"confidantOnline="+this.confidantOnline+","
	+"timeSpent="+timeSpent
	+"]";
};




var TunnelMgr=new _TunnelMgr();

