


function _UiMgr (uiMgrProps, handlerKeys) {	
	this.clientStorageKeyBytesKey="keyBytes"; // holds first 40K bytes (in hex, as 80K chars) of the image we use for authentication
	this.clientStorageKeyAccountCreatedForBytesKey="accountCreatedForBytesKey"; // the string "true" or "false" depending on whether the "keyBytes" value (previous line) has been used to create an account (if the user ditches the acct-creation process halfway through, "keyBytes" will be populated, but this will be "false")
	this.unregisteredConfidantIdentifier="Your&nbsp;Confidant";
	this.handlerKeys=handlerKeys; // handler key names
	this.sessionPathParamName=null;
	
	this.currentAccountHandle=null; // populated when user logs in
	this.currentAcctIsProfessionalClient=false; // true if professionalSite and the logged in acct is a client (not an originating acct)
	
	this.skinnedSiteData=null; //skinnedSiteData props: see EstablishSession handler for most current list

	this.tunnelIdCurrentlyEditingInSettings=null; // value of the tunnel we're currently editing in the settings dialog, if any
	
	// after Oct, 2014, we don't do temp PINs; we do longer temp passphrases acting as the "temp PIN"; 
	// but so we don't have to change the API, for back-compat we always pass this value (for newer-version 
	// clients) to tell the API that we're doing things the "new way".  This is, of course, an optional
	// param as far as the API goes, but for new clients, it is required for any calls that concern
	// temp PINS.  At some point, we can clean up this code, once we have forced a required client upgrade.
	// (Also: we actually pass this with EVERY ajax request, even though it's only required for some; easier.)
	this.tempPassphraseSecondHalfActsAsPin=true; // the name of this prop must match Constants.TEMP_PASSPHRASE_SECOND_HALF_ACTS_AS_PIN_PARAM_NAME
	
	this.currentAuthPage=null; // the currently visible authPage, if any (used in debugging, don't remove)
	
	this.hideFooterTextTimeout=null; // used to control delayed hiding of orange footer help text
	
	this.systemUsernameToken=null;
	this.systemUsernameTokenRegexp=null;
	this.unregisteredSecondUserNicknameToken=null;
	this.unregisteredSecondUserNicknameTokenRegexp=null;
	this.fileUploadUpdateIntervalMillis=null;
	
	this.lastCreatedTunnelClientId=null; // the id of the most recent tunnel created IN THIS SESSION, if any
	
	this.permanentlyDisableFooterLinks=false; // only ever set to true if this version of the app isn't going to work becasue it's too old
	
	this.maxConcurrentUploads=null;
	
	// Populated on login; for now, we check the value of this to see if we're logged in or not:
	this.acctPrefs={};
	
	/* - not used for now --
	this.debugUpdaterRequestsCount={
		mainUpdater:0,
		imageAuthUpdater:0,
		tunnelCreationUpdater:0,
		finalizeSecondUserUpdater:0
	};
	this.updaterDebugCodes=["mainUpdater", "imageAuthUpdater", "tunnelCreationUpdater", "finalizeSecondUserUpdater"];
	*/
	
	this.unalertedPageExitOkay=false;
	this.authPageIds=window.authUiPageCodes;
	this.nonAuthUiPageIds=["settings","about","help","privacy","tos","whatIsATunnel","betafeedback","betafeedbackthanks"];
	this.nonAuthUiPageIdsToLoadeds={settings:true, about:false, contact:false, help:false, privacy:false, tos:false, whatIsATunnel:false};
	this.nonAuthUiPageIdsToContents={};
	this.authPages={};
	for (var i=0; i<this.authPageIds.length; i++) {
		this.authPages[this.authPageIds[i]]=$("#"+this.authPageIds[i]);
	}
	
	// Following Map used to control the footer icons' visibility (the order matters and is left to right); keys are e.g. "settings", 
	// values are booleans indicating current visibility).  Visibility is controlled by UiMgr.showFooterIcon(key, visible) and 
	// UiMgr.hideAllFooterIcons().  Also note that some of these icons apply only
	// to the desktop site; attempting to show a nonexistent (on mobile) icon has no effect (and causes no errors).
	this._footerIconCodesToVisibilities=new Map();
	this._footerIconCodesToVisibilities.put("addTunnel", false);
	if (!window.mobileApp) {
		this._footerIconCodesToVisibilities.put("viewTunnelList", false); // note: this and the previous item are never visible at the same time
	}
	this._footerIconCodesToVisibilities.put("settings", false);
	this._footerIconCodesToVisibilities.put("help", false);
	this._footerIconCodesToVisibilities.put("exit", true);
	
	
	this.newTunnelSecondUserTempKey=null;
	this.newTunnelSecondUserTempPin=null;
	this.secondUserTempKey=null;
	this.secondUserTempPin=null;
	this.newTunnelCreatingUserNickname=null;
	this.newTunnelCreatingUserPin=null;
	this.secondUserPin=null;
	this.secondUserNickname=null;
	this.tempLoginKey=null;
	this.lastActivityTimeMillis=new Date().getTime();
	 
	
	
	// used in toggling document title on/off:
	this.toggleTitleInterval=null;
	this.toggleTitleIntervalLengthMillis=950;
	this.isTitleToggledOn=true;
	this.toggleTitleOffText="New Message...";
	this.toggleTitleOnText=$(document).attr("title");
	this.windowHasFocus=true;
	
	this.ignoreResizeEvent=false;
	this.ignoreResizeEventTimeout=null;
	
	this.minKeySizeKB=window.minKeySizeKB;
	
	
	this.confirmBeforeUiReset=false; // set to true when we want to confirm with the user whether they want to return to the homepage, resetting everything.
	
	this.mobileMultiSessionClientIdentifier=null; // used only by the mobile app.
	
	// now we take the uiMgrProps param passed to this constructor and add all its props (or replace as appropriate) to
	// this UiMgr:
	for (var prop in uiMgrProps) {
	    if (uiMgrProps.hasOwnProperty(prop)) {
	        this[prop]=uiMgrProps[prop];
	    }
	}
	
	this.currentMultipartMode=null; // possible values: "keyUpload", "mediaUpload"

	// used when we are using a clickable hidden input type=file element for uploads:
	this.transparentFormElHtml=null; // gets populated on first use, if above boolean is true


	/////////// V 2.0 new vars below; many of above may go away after:
	this.creatingAccount={
			creatingUserChosenImageKeyHexBytes:null, 
			creatingUserChosenPin:null,
			creatingUserChosenTunnelName:null,
			creatingUserChosenNickname:null,
			secondUserTempKey:null,
			secondUserSignupUrl:null,
			secondUserTempPin:null,
			creatingUserEmail:null, // used only for Tunnel T, etc.
			creatingUserFullname:null // used only for Tunnel T, etc.
	};
	this.imageAuthentication={
			chosenImageKeyHexBytes:null
	};
	
	// holds temp 2nd-user creds when logged in user is creating a new tunnel:
	this.newTunnelExistingAccount={
			secondUserTempKey:null,
			secondUserSignupUrl:null,
			secondUserTempPin:null			
	};
	
	// holds creds entered by logged-in second user entering a new passcode:
	this.newTunnelSecondUserExistingAccount={
			tempKey:null,
			tempPin:null			
	};
	
	this.newAccountSecondUser={
			chosenImageKeyHexBytes:null, // bytes of image which will authnticate this new user into her acct
			chosenPin:null, // PIN accompanying above image for auth
			chosenNickname:null, // nickname for the tunnel they've just joined
			tempKey:null, // the temp key given to the user by friend who asked them to join tunnel
			tempPin:null // temp PIN from friend
	};

	// holds creds while existing account adds a key to a new device:
	this.existingAccountNewDevice={
			chosenImageKeyHexBytes:null, // bytes of image which will authnticate this new user into her acct
			pin:null, // existing account PIN entered properly by user
			tempKey:null // temp passphrase issued to user on old device, entered properly on this new device
	};
	
	this.destroyingTunnel=false;
	
	this.hpCarouselInterval=null;
	this.hpCarouselInterval2=null;
	this.currentHpCarouselIdx=0;
	this.currentHpCarouselIdx2=0;
	this.numCarouselImages=window.numCarouselImages;
	this.numCarouselImages2=window.numCarouselImages2;
	this.hpCarouselIndicesToIds=[];
	for (var i=0; i<window.numCarouselImages; i++) {
		this.hpCarouselIndicesToIds[i]=i;
	}
	this.hpCarouselIndicesToIds2=[];
	for (var i=0; i<window.numCarouselImages2; i++) {
		this.hpCarouselIndicesToIds2[i]=i;
	}
	

}


var p=_UiMgr.prototype;

